/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CommStrategyUpTemplate.cpp
 *       Template for implementations of the communication
 *       protocol interface.
 *
 * All functions of the interface are offered
 * and will lead to an assertion.
 *
 */

#include <assert.h>
#include <mpi.h>
#include <stdio.h>
#include <pnmpimod.h>
#include <stdlib.h>
#include "CommStrategyUpTemplate.h"
#include "GtiMacros.h"

using namespace gti;

mGET_INSTANCE_FUNCTION(CommStrategyUpTemplate)
mFREE_INSTANCE_FUNCTION(CommStrategyUpTemplate)
mPNMPI_REGISTRATIONPOINT_FUNCTION(CommStrategyUpTemplate)

CommStrategyUpTemplate::CommStrategyUpTemplate (const char* instanceName)
    : ModuleBase<CommStrategyUpTemplate, I_CommStrategyUp> (instanceName)
{
    std::vector<I_Module*> subModInstances;
    subModInstances = createSubModuleInstances ();

    //A Comm Strategy either uses a regular communication protocol or a regular and a
    //fall-back one.
    //actual implementations should assert here
    ////assert ((subModInstances.size() == 1) || (subModInstances.size() == 2));

    if (protocol)
        protocol = (I_CommProtocol*) subModInstances[0];
    else
        protocol = NULL;

    if (subModInstances.size() == 2)
        fallback = (I_CommProtocol*) subModInstances[1];
    else
        fallback = NULL;
}

CommStrategyUpTemplate::~CommStrategyUpTemplate (void)
{
    /*Destroy sub modules*/
    if (protocol)
        destroySubModuleInstance ((I_Module*)protocol);
    protocol = NULL;

    if (fallback)
        destroySubModuleInstance ((I_Module*)fallback);
    fallback = NULL;
}

GTI_RETURN CommStrategyUpTemplate::shutdown (
        GTI_FLUSH_TYPE flush_behavior,
        GTI_SYNC_TYPE sync_behavior)
{
    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::send (
        void* buf,
        unsigned long num_bytes,
        void* free_data,
        GTI_RETURN (*buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
        )
{
    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::test (
        int* out_flag,
        unsigned long* out_num_bytes,
        void** out_buf,
        void** out_free_data,
        GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
        )
{
    *out_flag = false;

    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::wait (
        unsigned long* out_num_bytes,
        void** out_buf,
        void** out_free_data,
        GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
        )
{
    while (1==1) {}
    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::flush (
        void
        )
{
    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::enforce_blocking_comm (
        void
        )
{
    return GTI_SUCCESS;
}

GTI_RETURN CommStrategyUpTemplate::activate_fallback_comm (
        void
        )
{
    return GTI_SUCCESS;
}

/*EOF*/
