/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file Adjacency.h
 * 		@see gti::weaver::Adjacency
 *
 * @author Tobias Hilbrich
 * @date 13.01.2010
 */

#ifndef ADJACENCY_H
#define ADJACENCY_H

#include <string>
#include <vector>

#include "Level.h"
#include "Communication.h"
#include "Printable.h"

namespace gti
{
	namespace weaver
	{
		namespace layout
		{
			class Level;

			/**
			  * Simple class for a communication connection between
			  * two levels. It does specify in what relationship these
			  * levels are (e.g. who is the lower level).
			  */
			class Adjacency : virtual public Printable
			{
			public:

				/**
				 * Proper constructor.
				 * @param target pointer to target level of this comm.
				 * @param comm pointer to communication used.
				 */
				Adjacency (Level* target, Communication* comm);

				/**
				 * Invalid object constructor.
				 */
				Adjacency ();

				/**
				 * Destructor.
				 */
				~Adjacency ();

				/**
				 * Returns the target level.
				 * @return level.
				 */
				Level* getTarget (void);

				/**
				 * Returns the communication.
				 * @return communication.
				 */
				Communication* getComm (void);

				/**
				 * Hook method for printing,
				 * in order to enable the "<<" operator.
				 * @param out ostream to use.
				 * @return ostream after printing.
				 */
				virtual std::ostream& print (std::ostream& out) const;

			public:
				Level* target;
				Communication* comm;
			};
		} /*namespace layout*/
	} /*namespace weaver*/
} /*namespace must*/
#endif // ADJACENCY_H
